@@ -26,20 +26,22 @@ def main() -> None:
     )
     parser.add_argument(
         "--short",
-        nargs=1,
         type=bool,
         default=True,
         help="Shorten all strings above 20 characters",
     )
     parser.add_argument(
         "--format",
-        nargs=1,
         type=str,
         default="svg",
         help="File format of the generated cfg",
     )
     parser.add_argument(
-        "--calls", nargs=1, type=bool, default=True, help="Toggle call graph",
+        "--calls",
+        nargs=1,
+        type=bool,
+        default=True,
+        help="Toggle call graph",
     )
     parser.add_argument(
         "--show", nargs=1, type=bool, default=False, help="Open CFG after generation"
@@ -55,10 +57,14 @@ def main() -> None:
     # cfg.build_visual(cfg_name[:-3] + "_cfg", format="png", calls=True, show=False)
     # cfg.build_visual('controlflowgraph', format='png', calls=True, show=False)
     cfg.build_visual(
-        cfg_name[:-3] + "_cfg", format=args.format, calls=args.calls, show=args.show,
+        cfg_name[:-3] + "_cfg",
+        format=args.format,
+        calls=args.calls,
+        show=args.show,
     )
     # removes the CFG file, which maybe we could just turn off?
     os.remove(cfg_name[:-3] + "_cfg")
 
+
 if __name__ == "__main__":
     main()
