@@ -3,12 +3,16 @@
 """
 Control flow graph builder.
 """
-# Aurelien Coet, 2018.
 
 from typing import List, Optional
 import ast  # type: ignore
 from _ast import (
-    Import, Break, AnnAssign, For, ImportFrom, Assign,
+    Import,
+    Break,
+    AnnAssign,
+    For,
+    ImportFrom,
+    Assign,
     AugAssign,
     Call,
     ClassDef,
@@ -22,7 +26,7 @@ from _ast import (
     While,
     Yield,
 )  # type: ignore
-from py2cfg.model import Block, Link, CFG
+from py2cfg.model import Block, Link, CFG, FuncBlock
 
 
 def invert(node: Compare) -> Compare:
@@ -89,6 +93,7 @@ class CFGBuilder(ast.NodeVisitor):
 
     def __init__(self, short: bool = True) -> None:
         self.isShort = short
+        self._callbuf: FuncBlock = []
 
     # ---------- CFG building methods ---------- #
     def build(
@@ -160,6 +165,16 @@ class CFGBuilder(ast.NodeVisitor):
         self.current_id += 1
         return Block(self.current_id)
 
+    def new_func_block(self) -> FuncBlock:
+        """
+        Create a new function block with a new id.
+
+        Returns:
+            A FuncBlock object with a new unique id.
+        """
+        self.current_id += 1
+        return FuncBlock(self.current_id)
+
     def add_statement(self, block: Block, statement: stmt) -> None:
         """
         Add a statement to a block.
@@ -317,7 +332,26 @@ class CFGBuilder(ast.NodeVisitor):
 
         func = node.func
         func_name = visit_func(func)
-        self.current_block.func_calls.append(func_name)
+        if isinstance(node, ast.Call):
+            func_block = self.new_func_block()
+            self.add_statement(func_block, node)
+            func_block.name = func_name
+
+            if self._callbuf:
+                # Func block is argument of last block in self._callbuf
+                self._callbuf[-1].args.append(func_block)
+            else:
+                # Not inside argument context.
+                self.current_block.func_calls.append(func_name)
+                self.current_block.func_blocks.append(func_block)
+
+            self._callbuf.append(func_block)
+            for arg in node.args:
+                self.visit(arg)
+            top = self._callbuf.pop()
+            assert hash(top) == hash(func_block)
+        else:
+            self.current_block.func_calls.append(func_name)
 
     def visit_Assign(self, node: Assign) -> None:
         self.add_statement(self.current_block, node)
@@ -360,7 +394,8 @@ class CFGBuilder(ast.NodeVisitor):
         else:
             # Add the If statement at the end of the current block.
             self.add_statement(self.current_block, node)
-
+        if any(isinstance(node.test, T) for T in (ast.Compare, ast.Call)):
+            self.visit(node.test)
         # Create a new block for the body of the if. (storing the True case)
         if_block = self.new_block()
 
@@ -395,6 +430,8 @@ class CFGBuilder(ast.NodeVisitor):
         self.current_block = loop_guard
         self.add_statement(self.current_block, node)
 
+        if isinstance(node.test, ast.Call):
+            self.visit(node.test)
         # New block for the case where the test in the while is True.
         while_block = self.new_block()
         self.add_exit(self.current_block, while_block, node.test)
@@ -417,6 +454,8 @@ class CFGBuilder(ast.NodeVisitor):
         self.current_block = loop_guard
         self.add_statement(self.current_block, node)
 
+        if isinstance(node.iter, ast.Call):
+            self.visit(node.iter)
         # New block for the body of the for-loop.
         for_block = self.new_block()
         self.add_exit(self.current_block, for_block, node.iter)
