@@ -3,9 +3,8 @@
 """
 Control flow graph for Python programs.
 """
-# Aurelien Coet, 2018.
 
-from typing import Any, Tuple, List, Optional, Iterator
+from typing import Any, Tuple, List, Optional, Iterator, Set
 import ast  # type: ignore
 from _ast import Compare  # For type-hinting
 import astor  # type: ignore
@@ -23,21 +22,29 @@ class Block(object):
     a list of Links that represent control flow jumps.
     """
 
-    # Look into ctypes.Structure
-    __slots__ = ["id", "statements", "func_calls", "predecessors", "exits"]
+    __slots__ = [
+        "id",
+        "statements",
+        "func_calls",
+        "predecessors",
+        "exits",
+        "func_blocks",
+    ]
 
     def __init__(self, id: int) -> None:
         # Id of the block.
         self.id = id
         # Statements in the block.
-        self.statements = []
+        self.statements: List[ast.AST] = []
         # Calls to functions inside the block (represents context switches to
         # some functions' CFGs).
-        self.func_calls = []
+        self.func_calls: List[str] = []
         # Links to predecessors in a control flow graph.
-        self.predecessors = []
+        self.predecessors: List[Link] = []
         # Links to the next blocks in a control flow graph.
-        self.exits = []
+        self.exits: List[Link] = []
+        # Function blocks within this block
+        self.func_blocks: List[FuncBlock] = []
 
     def __str__(self) -> str:
         if self.statements:
@@ -80,7 +87,7 @@ class Block(object):
         src = ""
         for statement in self.statements:
             if type(statement) in [ast.If, ast.For, ast.While]:
-                src += (astor.to_source(statement)).split("\n")[0] + "\n"
+                src += astor.to_source(statement).split("\n")[0] + "\n"
             elif (
                 type(statement) == ast.FunctionDef
                 or type(statement) == ast.AsyncFunctionDef
@@ -103,10 +110,15 @@ class Block(object):
             txt += func_name + "\n"
         return txt
 
-    def type(self, default: None=None) -> Any:
-        default = ast.Pass if default is None else default
+    def type(self, default: Optional[Any] = None) -> Any:
+        default = ast.AST if default is None else default
         return type(self.statements[0]) if self.statements else default
 
+    def __hash__(self):
+        # Allows Block objects to be used as dict keys and set elements
+        # return id(self)
+        return self.id
+
 
 class Link(object):
     """
@@ -120,7 +132,10 @@ class Link(object):
     __slots__ = ["source", "target", "exitcase"]
 
     def __init__(
-        self, source: Block, target: Block, exitcase: Optional[ast.Compare] = None,
+        self,
+        source: Block,
+        target: Block,
+        exitcase: Optional[ast.Compare] = None,
     ) -> None:
         assert isinstance(target, Block), "Source of a link must be a block"
         assert isinstance(target, Block), "Target of a link must be a block"
@@ -154,6 +169,13 @@ class Link(object):
         return ""
 
 
+class FuncBlock(Block):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.args: List[FuncBlock] = []
+        self.name: str = None
+
+
 class CFG(object):
     """
     Control flow graph (CFG).
@@ -165,6 +187,7 @@ class CFG(object):
     """
 
     # Also serves as graph Key table
+    # TODO Change value type to dict. Can be upacked into graph.node fn.
     node_styles = {
         "input": ("parallelogram", "#afeeee"),  # Pale Turquoise
         "default": ("rectangle", "#FFFB81"),  # Pale Yellow
@@ -176,6 +199,7 @@ class CFG(object):
     }
     DEFAULT = node_styles["default"]
 
+    
     def __init__(self, name: str, asynchr: bool = False, short: bool = True) -> None:
         assert isinstance(name, str), "Name of a CFG must be a string"
         assert isinstance(asynchr, bool), "Async must be a boolean value"
@@ -185,13 +209,13 @@ class CFG(object):
         # program is considered as a synchronous function (main).
         self.asynchr = asynchr
         # Entry block of the CFG.
-        self.entryblock = None
+        self.entryblock: Optional[Block] = None
         # Final blocks of the CFG.
-        self.finalblocks = []
+        self.finalblocks: List[Block] = []
         # Sub-CFGs for functions defined inside the current CFG.
-        self.functioncfgs = {}
+        self.functioncfgs: dict = {}
         # Sub-CFGs
-        self.classcfgs = {}
+        self.classcfgs: dict = {}
         self.isShort = short
 
     def __str__(self) -> str:
@@ -211,7 +235,11 @@ class CFG(object):
                     if isinstance(typeobj, type)
                     else str(typeobj)
                 ),
-                _attributes={"shape": shape, "style": "filled", "fillcolor": color,},
+                _attributes={
+                    "shape": shape,
+                    "style": "filled",
+                    "fillcolor": color,
+                },
             )
 
         # Adding edge helps with alignment.
@@ -233,20 +261,29 @@ class CFG(object):
     def _edge_handler(self, link: Link) -> Tuple[None, str, str]:
         return None, "black", link.get_exitcase().strip()
 
-    def stylize_node(self, block: Block, default: None=None) -> tuple:
+    def stylize_node(self, block: Block, default: None = None) -> tuple:
         # Since we're modelling ast objects, we might as well copy how they
         # implement handlers. This should allow us to implement new node types
         # simply defining a method `node_ASTNAME`
         typeobject = block.type(default)
         name = typeobject.__name__ if isinstance(typeobject, type) else typeobject
-        style = getattr(self, f"node_{name}", self._style_handler,)(block, typeobject)
+        style = getattr(
+            self,
+            f"node_{name}",
+            self._style_handler,
+        )(block, typeobject)
         assert len(style) == 3, "Style handler must return (shape, color, label)"
         assert all(
             isinstance(attr, str) or attr or attr is None for attr in style
         ), "Style attributes must be str or NoneType"
         return style
 
-    def stylize_edge(self, link: Link, default: None=None, default_target: None=None,) -> tuple:
+    def stylize_edge(
+        self,
+        link: Link,
+        default: None = None,
+        default_target: None = None,
+    ) -> tuple:
         typeobject = link.source.type(default)
         name = typeobject.__name__ if isinstance(typeobject, type) else typeobject
         target_type = link.target.type(default_target)
@@ -266,13 +303,49 @@ class CFG(object):
         ), "style attributes must be str or NoneType"
         return style
 
+    def _visit_func(
+        self,
+        graph: Digraph,
+        block: FuncBlock,
+        last: str = None,
+        visited: Set[Block] = set(),
+    ) -> None:
+        if block in visited:
+            return
+        visited.add(block)
+        shape, color, _ = self.stylize_node(block)
+        graph.node(
+            str(block.id),
+            label=block.name,
+            _attributes={
+                "shape": shape,
+                "color": color,
+                "style": "filled",
+            },
+        )
+        if last is not None:
+            graph.edge(
+                last,
+                str(block.id),
+                _attributes={
+                    "color": "black",
+                },
+            )
+        for arg in block.args:
+            self._visit_func(graph, arg, str(block.id))
+
     def _visit_blocks(
-        self, graph: Digraph, block: Block, visited: List[int] = [], calls: bool = True,
+        self,
+        graph: Digraph,
+        block: Block,
+        visited: Set[Block] = set(),
+        calls: bool = True,
+        format: str = None,
     ) -> None:
         # Don't visit blocks twice.
-        if block.id in visited:
+        if block in visited:
             return
-        visited.append(block.id)
+        visited.add(block)
         nodeshape, nodecolor, nodelabel = self.stylize_node(block)
         node_type = block.type()
         original_nodelabel = nodelabel
@@ -306,18 +379,48 @@ class CFG(object):
             },
         )
 
-        # Show the block's function calls in a node.
-        # TODO(jstuder) Include conditional blocks in "calls" edges
         if calls and block.func_calls:
             calls_node = str(block.id) + "_calls"
+
             # Remove any duplicates by splitting on newlines and creating a set
             calls_label = block.get_calls().strip()
+
+            # Create a new subgraph for call statement
+            calls_subgraph = gv.Digraph(
+                name=f"cluster_{block.id}",
+                format=format,
+                graph_attr={
+                    "rankdir": "TB",
+                    "ranksep": "0.02",
+                    "style": "filled",
+                    "color": "purple",
+                    "compound": "true",
+                    "shape": self.node_styles[ast.Call][0],
+                    "label": "",
+                },
+            )
+            # Generate control flow edges for function arguments
+            for func_block in block.func_blocks:
+                    graph.edge(
+                        str(block.id),
+                        str(func_block.id),
+                        label="calls",
+                        _attributes={"style": "dashed"},
+                    )
+                    self._visit_func(
+                        calls_subgraph,
+                        func_block,
+                        visited=set(),
+                    )
+            graph.subgraph(calls_subgraph)
+
             tmp = ""
             for line in calls_label.splitlines():
                 if "input" in line:
+                    input_node = str(block.id) + "_input"
                     nodeshape, nodecolor = self.node_styles["input"]
                     graph.node(
-                        str(block.id) + "_input",
+                        input_node,
                         label=line,
                         _attributes={
                             "style": "filled",
@@ -325,30 +428,18 @@ class CFG(object):
                             "fillcolor": nodecolor,
                         },
                     )
-                    graph.edge(str(block.id) + "_input", str(block.id))
+                    graph.edge(input_node, str(block.id))  # yellow
                     # _attributes={'style': 'dashed'})
+
                 else:
                     line += "\l"
                     tmp += line
-            calls_label = tmp
-            shape, color = self.node_styles[ast.Call]
-            graph.node(
-                calls_node,
-                label=calls_label,
-                _attributes={"shape": shape, "style": "filled", "fillcolor": color,},
-            )
 
-            graph.edge(
-                str(block.id),
-                calls_node,
-                label="calls",
-                _attributes={"style": "dashed"},
-            )
 
         # Recursively visit all the blocks of the CFG.
         for exit in block.exits:
             assert block == exit.source
-            self._visit_blocks(graph, exit.target, visited, calls=calls)
+            self._visit_blocks(graph, exit.target, visited, calls=calls, format=format)
             edgeshape, edgecolor, edgelabel = self.stylize_edge(exit)
             graph.edge(
                 str(block.id),
@@ -361,13 +452,23 @@ class CFG(object):
         graph = gv.Digraph(
             name="cluster" + self.name,
             format=format,
-            graph_attr={"label": self.name, "rankdir": "TB", "ranksep": "0.02"},
+            graph_attr={
+                "label": self.name,
+                "rankdir": "TB",
+                "ranksep": "0.02",
+                "compound": "True",
+            },
+        )
+        self._visit_blocks(
+            graph, self.entryblock, visited=set(), calls=calls, format=format
         )
-        self._visit_blocks(graph, self.entryblock, visited=[], calls=calls)
         # Build the subgraphs for the function definitions in the CFG and add
         # them to the graph.
         for subcfg in self.classcfgs:
-            subgraph = self.classcfgs[subcfg]._build_visual(format=format, calls=calls,)
+            subgraph = self.classcfgs[subcfg]._build_visual(
+                format=format,
+                calls=calls,
+            )
             graph.subgraph(subgraph)
 
         for subcfg in self.functioncfgs:
@@ -402,7 +503,7 @@ class CFG(object):
         recursively yields from any sub graphs
         """
         visited = set()
-        to_visit = [self.entryblock]
+        to_visit: List[Block] = [self.entryblock]
 
         while to_visit:
             block = to_visit.pop(0)
@@ -446,3 +547,5 @@ class CFG(object):
         if edgelabel in nodelabel:
             color = "green"
         return None, color, edgelabel
+
+
